using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using MessengerAPI;
using GroupLab.Phidgets;


namespace MSNPhidgets
{
	/// <summary>
	/// Use the MSN Messenger API COM module to control the interface
	/// with phidget components.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		private GroupLab.Phidgets.UI.PhidgetManagerSkin phidgetManagerSkin1;
		private System.ComponentModel.Container components = null;

		private Messenger msn;
		private TextLCD textLCD;
		private InterfaceKit interfaceKit;
		private System.Windows.Forms.Button button1;
		private RFID rfid;
		private System.Windows.Forms.Label label1;

		private Hashtable onlineStates = new Hashtable();


		/// <summary>
		/// Constructor
		/// </summary>
		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// Initialize the MSN Messenger COM object, and add
			// two event callbacks: listen for changes of the users status change
			// and also listen for changes of the own status change.
			//
			msn = new Messenger();
			msn.OnContactStatusChange +=new DMessengerEvents_OnContactStatusChangeEventHandler(msn_OnContactStatusChange);
			msn.OnMyStatusChange +=new DMessengerEvents_OnMyStatusChangeEventHandler(msn_OnMyStatusChange);

			//
			// Initialize the phidget RFID, and add event callback listener
			// for detected RFID tags.
			//
			this.rfid = new RFID();
			this.rfid.AutoAttach = true;
			this.rfid.Tag +=new RFIDTagEventHandler(rfid_Tag);

			//
			// New interface kit phidget
			//
			this.interfaceKit = new InterfaceKit();
			this.interfaceKit.AutoAttach = true;
			this.interfaceKit.Attach +=new EventHandler(interfaceKit_Attach);

			//
			// TextLCD phidget, to display the users status change
			//
			this.textLCD = new TextLCD();
			this.textLCD.AutoAttach = true;
			this.textLCD.Attach +=new EventHandler(textLCD_Attach);


			this.initOnlineStatesHashtable();
		}


		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.phidgetManagerSkin1 = new GroupLab.Phidgets.UI.PhidgetManagerSkin();
			this.button1 = new System.Windows.Forms.Button();
			this.label1 = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// phidgetManagerSkin1
			// 
			this.phidgetManagerSkin1.Location = new System.Drawing.Point(8, 136);
			this.phidgetManagerSkin1.Name = "phidgetManagerSkin1";
			this.phidgetManagerSkin1.Size = new System.Drawing.Size(528, 176);
			this.phidgetManagerSkin1.TabIndex = 0;
			// 
			// button1
			// 
			this.button1.Location = new System.Drawing.Point(16, 80);
			this.button1.Name = "button1";
			this.button1.Size = new System.Drawing.Size(168, 40);
			this.button1.TabIndex = 1;
			this.button1.Text = "Exit Application";
			// 
			// label1
			// 
			this.label1.Font = new System.Drawing.Font("Arial", 11.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label1.Location = new System.Drawing.Point(16, 16);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(312, 48);
			this.label1.TabIndex = 2;
			this.label1.Text = "Using Phidget components to control Microsoft MSN Messenger.";
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(544, 328);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.button1);
			this.Controls.Add(this.phidgetManagerSkin1);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.Name = "Form1";
			this.Text = "MSN Phidgets";
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}


		/// <summary>
		/// Callback event listener: is called, when users in the contact list
		/// change their online state
		/// </summary>
		/// <param name="pMContact">Contact</param>
		/// <param name="mStatus">Status</param>
		private void msn_OnContactStatusChange(object pMContact, MISTATUS mStatus)
		{
			MessengerAPI.IMessengerContact contact = (IMessengerContact)pMContact;
			
			//
			// If Phidget TextLCD is attached, change the message of the display.
			//
			if(this.textLCD.Attached)
			{
				this.textLCD.Display = contact.FriendlyName + ": " + this.getStatusText(mStatus);
			}


		}

		/// <summary>
		/// Callback event listener: is called, when the local user 
		/// changes her/his online state
		/// </summary>
		/// <param name="hr"></param>
		/// <param name="mMyStatus"></param>
		private void msn_OnMyStatusChange(int hr, MISTATUS mMyStatus)
		{
			//
			// If the status is ONLINE, then activate the green LED,
			// otherwise activate the red LED.
			//
			if(mMyStatus == MessengerAPI.MISTATUS.MISTATUS_ONLINE)
			{
				this.interfaceKit.Outputs[0].State = true;
				this.interfaceKit.Outputs[1].State = false;
			} 
			else 
			{
				this.interfaceKit.Outputs[0].State = false;
				this.interfaceKit.Outputs[1].State = true;
			}
		}



		/// <summary>
		/// Event callback for detected RFID tags. Change the online state
		/// of the user in dependance of the detected RFID tag.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e">Tag information</param>
		private void rfid_Tag(object sender, RFIDTagEventArgs e)
		{
			if(e.Tag == "0102ac71ee")
			{
				if(!(this.msn.MyStatus == MessengerAPI.MISTATUS.MISTATUS_ONLINE))
				{
					this.msn.MyStatus = MessengerAPI.MISTATUS.MISTATUS_ONLINE;
				}
			} 
			else if (e.Tag == "0102ac7191")
			{
				if(!(this.msn.MyStatus == MessengerAPI.MISTATUS.MISTATUS_AWAY))
				{
					this.msn.MyStatus = MessengerAPI.MISTATUS.MISTATUS_AWAY;
				}
			}
		}


		/// <summary>
		/// Is called, when a Phidget TextLCd is attached.
		/// Initialize the display, and change the current displayed text.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void textLCD_Attach(object sender, EventArgs e)
		{
			this.textLCD.Backlight = true;
			this.textLCD.CursorBlink = false;
			this.textLCD.Display = "MSN Phidget";
		}

		/// <summary>
		/// Callback for attached interface kit
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void interfaceKit_Attach(object sender, EventArgs e)
		{
			if(this.msn.MyStatus == MessengerAPI.MISTATUS.MISTATUS_ONLINE)
			{
				this.interfaceKit.Outputs[0].State = true;
				this.interfaceKit.Outputs[1].State = false;
			} 
			else 
			{
				this.interfaceKit.Outputs[0].State = false;
				this.interfaceKit.Outputs[1].State = true;
			}
		}



		/// <summary>
		/// Initialize the hashtable with the string translations of the 
		/// messenger online states.
		/// </summary>
		private void initOnlineStatesHashtable()
		{
			this.onlineStates.Add(MISTATUS.MISTATUS_AWAY,			"Away");
			this.onlineStates.Add(MISTATUS.MISTATUS_BE_RIGHT_BACK,	"Be right back");
			this.onlineStates.Add(MISTATUS.MISTATUS_BUSY,			"Busy");
			this.onlineStates.Add(MISTATUS.MISTATUS_IDLE,			"Idle");
			this.onlineStates.Add(MISTATUS.MISTATUS_OFFLINE,		"Offline");
			this.onlineStates.Add(MISTATUS.MISTATUS_ON_THE_PHONE,	"On the phone");
			this.onlineStates.Add(MISTATUS.MISTATUS_ONLINE,			"Online");
			this.onlineStates.Add(MISTATUS.MISTATUS_OUT_TO_LUNCH,	"Out to lunch");
		}


		/// <summary>
		/// Returns the text description for messenger status
		/// </summary>
		/// <param name="mMyStatus">The messenger status</param>
		/// <returns>Status as string</returns>
		private string getStatusText(MISTATUS mMyStatus)
		{
			if(this.onlineStates.ContainsKey(mMyStatus))
			{
				return (string)this.onlineStates[mMyStatus];
			} 
			else 
			{
				return "Unknown";
			}
		}


	}
}
